@@ -8,12 +8,8 @@ from uuid import UUID, uuid4
 
 from codeboxapi import CodeBox  # type: ignore
 from codeboxapi.schema import CodeBoxOutput  # type: ignore
-from langchain.agents import (
-    AgentExecutor,
-    BaseSingleActionAgent,
-    ConversationalAgent,
-    ConversationalChatAgent,
-)
+from langchain.agents import (AgentExecutor, BaseSingleActionAgent,
+                              ConversationalAgent, ConversationalChatAgent)
 from langchain.chat_models import AzureChatOpenAI, ChatAnthropic, ChatOpenAI
 from langchain.chat_models.base import BaseChatModel
 from langchain.memory import ConversationBufferMemory
@@ -23,7 +19,7 @@ from langchain.memory.chat_message_histories import (
     RedisChatMessageHistory,
 )
 from langchain.prompts.chat import MessagesPlaceholder
-from langchain.schema import BaseChatMessageHistory, BaseLanguageModel
+from langchain.schema import BaseChatMessageHistory, BaseLanguageModel, SystemMessage
 from langchain.tools import BaseTool, StructuredTool
 
 from codeinterpreterapi_hyperobjekt.agents import OpenAIFunctionsAgent
@@ -50,6 +46,8 @@ class CodeInterpreterSession:
     def __init__(
         self,
         llm: Optional[BaseLanguageModel] = None,
+        system_message: SystemMessage = code_interpreter_system_message,
+        max_iterations: int = 9,
         additional_tools: list[BaseTool] = [],
         **kwargs,
     ) -> None:
@@ -57,6 +55,8 @@ class CodeInterpreterSession:
         self.verbose = kwargs.get("verbose", settings.VERBOSE)
         self.tools: list[BaseTool] = self._tools(additional_tools)
         self.llm: BaseLanguageModel = llm or self._choose_llm(**kwargs)
+        self.max_iterations = max_iterations
+        self.system_message = system_message
         self.agent_executor: Optional[AgentExecutor] = None
         self.input_files: list[File] = []
         self.output_files: list[File] = []
@@ -97,19 +97,19 @@ class CodeInterpreterSession:
             ),
         ]
 
-    def _choose_llm(
-        self, model: str = "gpt-4", openai_api_key: Optional[str] = None, **kwargs
+    def _llm(
+        self, model: Optional[str] = None, openai_api_key: Optional[str] = None
     ) -> BaseChatModel:
-        if "gpt" in model:
-            openai_api_key = (
-                openai_api_key
-                or settings.OPENAI_API_KEY
-                or getenv("OPENAI_API_KEY", None)
+        if model is None:
+            model = "gpt-4"
+
+        if openai_api_key is None:
+            raise ValueError(
+                "OpenAI API key missing. Set OPENAI_API_KEY env variable or pass `openai_api_key` to session."
             )
             if openai_api_key is None:
                 raise ValueError(
-                    "OpenAI API key missing. Set OPENAI_API_KEY env variable "
-                    "or pass `openai_api_key` to session."
+                    "OpenAI API key missing. Set OPENAI_API_KEY env variable or pass `openai_api_key` to session."
                 )
             openai_api_version = getenv("OPENAI_API_VERSION")
             openai_api_base = getenv("OPENAI_API_BASE")
@@ -148,7 +148,7 @@ class CodeInterpreterSession:
             OpenAIFunctionsAgent.from_llm_and_tools(
                 llm=self.llm,
                 tools=self.tools,
-                system_message=code_interpreter_system_message,
+                system_message=self.system_message,
                 extra_prompt_messages=[
                     MessagesPlaceholder(variable_name="chat_history")
                 ],
@@ -189,7 +189,7 @@ class CodeInterpreterSession:
     def _agent_executor(self) -> AgentExecutor:
         return AgentExecutor.from_agent_and_tools(
             agent=self._choose_agent(),
-            max_iterations=9,
+            max_iterations=self.max_iterations,
             tools=self.tools,
             verbose=self.verbose,
             memory=ConversationBufferMemory(
@@ -277,13 +277,10 @@ class CodeInterpreterSession:
                     r"ModuleNotFoundError: No module named '(.*)'", output.content
                 ):
                     await self.codebox.ainstall(package.group(1))
-                    return (
-                        f"{package.group(1)} was missing but "
-                        "got installed now. Please try again."
-                    )
+                    return f"{package.group(1)} was missing but got installed now. Please try again."
             else:
-                # TODO: preanalyze error to optimize next code generation
                 pass
+            # TODO: preanalyze error to optimize next code generation
             if self.verbose:
                 print("Error:", output.content)
 
